@@ -4,7 +4,7 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
                              net = F, n.linear = n.linear,  
                              sm.names, lin.names, lin.means, 
                              method, verbose, 
-                             Pwee.list = Pwee.list, max.df){
+                             Pwee.list = Pwee.list, fixed.df){
  
   
   
@@ -16,8 +16,10 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
   stretch_index<-vector("numeric", length = n.smooth.comps)
   for(i in 1:length(P.list)) stretch_index[i]<-ifelse(class(P.list[[i]]) == "list", length(P.list[[i]]), 1)
   # unnest the list of penalty matrices
-  P.flat    <- make_flat(P.list)
-  Pwee.flat <- make_flat(Pwee.list)
+  # P.flat    <- make_flat(P.list)
+  P.flat    <- unlist(P.list, use.names = F)
+  # Pwee.flat <- make_flat(Pwee.list)
+  Pwee.list   <- unlist(Pwee.list, use.names = F)
   # the total number of smoothing parameters (=sum(stretch_index))
   n.smooth.pars      <-length(P.flat)
   # length of the response vector
@@ -34,42 +36,37 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
   
   # IDENTIFIABILITY CONSTRAINTS USED IN ESTIMATION OF UNIVARIATE SMOOTHS
   # ------------------------------------------------------------------------------
-  blockzero<-lapply(X.dim, make_sparse)
-  idList <- XTX.list <- lapply(X.list, crossprodspam)
-  makezero<-n.linear + 1
-  # zero out the identifiability constraint associated with linear components, 
+  blockzero  <- lapply(X.dim, make_sparse)
+  idList     <- lapply(X.list, crossprodspam)
+  # zero out the identifiability constraint associated with any linear components, 
   #  and include those associated with standard spline smooths
-  idList <- XTX.list
-  ridgeM <- list()
-  for(z in 1:makezero) idList[[z]] <- blockzero[[z]] 
-  for(term in 1:(n.terms - net)) ridgeM[[term]]<-(10^-8)*get_block_penalty(P = idList[[term]], blockzero = blockzero, i=term)
-  ridgeM <- Reduce("+", ridgeM)
-  
-  # IDENTIFIABILITY CONSTRAINTS USED FOR THE NETWORK COMPONENT, REGARDLESS OF "ridge".
-  # THIS MATRIX IS FIXED THROUGHOUT, AND NO PARAMETER IS ASSOCIATED WITH IT.
-  # ------------------------------------------------------------------------------
-  nseg          <- inds[n.terms]
-  net_par_start <- np - nseg + 1
-  net_pars      <- net_par_start:np
-  if(net)          diag.spam(ridgeM)[net_pars] <- 10^-8
+  ridgeM     <- list()
+  for(z in 1:(n.linear + 1)) idList[[z]] <- blockzero[[z]] 
+  for(term in 1:(n.terms - net)){
+    ridgeM[[term]]<- get_block_penalty(P = idList[[term]], blockzero = blockzero, i = term)*(10^-6)
+  }
+  ridgeM       <- Reduce("+", ridgeM)
+  # P.flat       <- c(P.flat, ridgeM)
+
 
   # CHOLESKY FACTORISATION, Xy.
   # ------------------------------------------------------------------------------
   P          <- P.flat
   Xy         <- t(X.spam) %*% response
-  cholFactor <- chol.spam(XTX.spam + Reduce("+", P.flat) + ridgeM, pivot = "MMD", eps = 10^-6)
+  cholFactor <- chol.spam(XTX.spam + Reduce("+", P.flat) + ridgeM, pivot = "MMD", eps = 10^-8)
     
   # WORK OUT THE INITIAL VALUES OF THE SMOOTHING PARAMETERS FOR THE OPTIMISER
   # ------------------------------------------------------------------------------
   if(is.numeric(control$start)){
     start.vals <- control$start
   } else if(is.null(control$start)){
-    start.vals <- rep(0, n.smooth.pars)
+    start.vals <- rep(1, n.smooth.pars)
   }
   
   
   if(method %in% c("AICC", "GCV", "AIC")){
       if(is.numeric(control$approx)){
+        set.seed(1)
         w <- matrix(sample(c(-1,1), n*control$approx, replace = T), nrow=n, ncol=control$approx)
         Xw<-t(X.spam) %*% w
         objective<-function(rhoArgs){
@@ -79,6 +76,7 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
                           n.sm = n.smooth.pars, crit = method, identifyBit = ridgeM)
         }
       } else {
+        Xw<-t(X.spam)
         objective<-function(rhoArgs){
           get_crit_exact(rhoArgs, X = X.spam, 
                          XTX = XTX.spam, P = P, response = response,
@@ -88,32 +86,37 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
       }
       if(length(P) == 1){
         a1       <- proc.time()
-        optimal  <- optimize(f = objective, upper = 20, lower = -20, tol = control$tol)
+        optimal  <- optimize(f = objective, upper = 30, lower = -30, tol = control$tol)
         a2       <- proc.time()
         rhoOptim <- optimal$minimum
       }
       if(length(P) > 1){
-        lower <- rep(-20, length(P.flat))
-        upper <- rep(20, length(P.flat))
-        if(net){
-          if(is.null(max.df)){
-            lower[length(upper) - 1] <- get_max_df(P.flat, rhoInput = rep(-10, length(P.flat)), ridgeM, XTX.spam, 
-                                                   cholFactor, info, Xy, X.spam, X.list, max.df)
-          } else {
-            lower[length(upper) - 1] <- 0
-          }
-          start.vals[length(start.vals) - 1] <- lower[length(upper) - 1] + 1
-          start.vals[length(start.vals)]     <- -10
-          lower[length(start.vals)]          <- -20
-          upper[length(start.vals)]          <-  0
-        }
-          a1       <- proc.time()
-          optimal  <- optim(par = start.vals, fn = objective, method = "Nelder-Mead", control=list(reltol = 10^-8, maxit = 500))
-          a2       <- proc.time()
+        a1       <- proc.time()
+        if( ((length(P) > 2) & (!is.null(fixed.df))) | ((length(P) > 1) & (is.null(fixed.df)))){
+          # get the optimal smoothing parameters
+          optimal  <- optim(par = start.vals, fn = objective, method = control$optim.method, 
+                            control = list(trace = control$trace, maxit = control$maxit,
+                                           reltol = control$tol))
           rhoOptim <- optimal$par
-      } 
-      ntime   <- round(abs(a1-a2)[3], 3)
-      nits    <- ifelse(length(P) > 1, optimal$counts[1], 1)
+        } else {
+          rhoOptim <- start.vals
+        }
+        # if fixed.df is specified (only happens if a network smooth is specified)
+        # then search for smoothing parameter values that satisfy this df level 
+        # given previous optimal values for other smooth terms
+          if(!is.null(fixed.df)){
+            approxTF <- is.numeric(control$approx)
+            optimal  <- get_fixed_df(P.flat, rhoInput = rhoOptim, 
+                                     ridgeM, XTX.spam, cholFactor, info, 
+                                     Xy, Xw = Xw, y = response, X.spam, 
+                                     approx = approxTF,
+                                     X.list, fixed.df, maxit = control$maxit)
+            rhoOptim[(length(rhoOptim) - 1):length(rhoOptim)] <- optimal$par
+          }
+        a2       <- proc.time()
+      }
+    ntime   <- round(abs(a1-a2)[3], 3)
+    nits    <- ifelse(length(P) > 1, optimal$counts[1], 1)
   }
 
   # FINALLY FIT THE MODEL USING THE ESTIMATED SMOOTHING PARAMETERS
@@ -131,7 +134,7 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
   pdof     <- vector("numeric", length = n.smooth.comps)
   if(is.numeric(control$approx)){
     left1 <- forwardsolve.spam(U, Xw) 
-    ED1   <- sum(rowMeans(left1*left1))
+    ED1   <- ifelse(!is.null(dim(left1)), sum(rowMeans(left1*left1)), sum(left1*left1))
     left2 <- backsolve.spam(U, left1)
     left3 <- X.spam %*% left2
     ED2   <- sum(rowMeans(left3*left3))
@@ -141,7 +144,7 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
       zero_out <- which(!(1:nrow(Xw)) %in% (cum.inds[j-1]+1):(cum.inds[j]))
       Xw_zero[zero_out,] <- 0
       right1    <- forwardsolve.spam(U, Xw_zero) 
-      pdof[i]   <- sum(rowMeans(right1 * left1))
+      pdof[i]   <-  ifelse(!is.null(dim(left1)), sum(rowMeans(right1 * left1)), sum((right1 * left1)))
     }  
   } else {
     Xw    <- t(X.spam)
@@ -164,7 +167,6 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
   # ------------------------------------------------------------------------------
   sigma.sq    <- sum((response - fit)^2)/(n - 2*ED1 + ED2) # model variance
   dof.smooth  <- rep(pdof, stretch_index)
-  allrho      <- exp(rhoOptim)
   diagnostics <- list(ED = 2*ED1 - ED2, sigma.sq = sigma.sq, fit = fit)
   
   # IF VERBOSE IS SET TO TRUE, PRINT THE CONVERGENCE STATUS FROM OPTIMISER
@@ -177,9 +179,16 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
       } else { cat("Warning, convergence was not reached \n\n")}   
     }
   } else if(length(P.flat) == 1) cat(paste("Convergence reached after", ntime, "s \n\n"))
+
   
   # PRINT THE SMOOTHING PARAMETERS, AND ASSOCIATED EFFECTIVE DIMENSIONS
   # ------------------------------------------------------------------------------
+  if(!is.null(fixed.df)){
+    if(method == "AICC") out <- log(sum((response - fit)^2)) + (2*(ED1+1)/(n-ED1-2))
+    if(method == "AIC")  out <- log(sum((response - fit)^2)) + 2*ED1/n
+    if(method == "GCV")  out <- sum((response - fit)^2)/(1-(ED1/n))^2
+  }
+  
   get.inner.length  <- function(L) ifelse(!class(L) == "list", 1, length(L))
   inner.length      <- lapply(P.list, get.inner.length)
   tablecol          <- max(unlist(inner.length))
@@ -188,7 +197,7 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
   increment         <- 1
   for(i in 1:length(P.list)){
     for(j in 1:inner.length[[i]]){
-      retPrint[i,j]  <- ret[i,j] <- round(log(allrho[increment]), 2)
+      retPrint[i,j]  <- ret[i,j] <- round(rhoOptim[increment], 2)
       increment      <- increment + 1
     }
   }
@@ -205,11 +214,11 @@ get_optimal_smooth<-function(P.list, X.spam, XTX.spam, X.list,
     cat("------------------------------------------ \n")
     print(as.data.frame(retPrint)) 
   }
-  function_min <- ifelse(length(P.flat) == 1, round(optimal$objective, 1), round(optimal$value, 1))
+  function_min <- ifelse(length(P.flat) == 1, round(optimal$objective, 3), round(optimal$value, 3))
   cat(paste("\n\n", method, " = ", function_min, sep = ""))
   # RETURN MODEL OUTPUT
   # ------------------------------------------------------------------------------
-  list(pars = allrho, beta_hat = beta_hat, ED = 2*ED1 - ED2, fit = fit, 
+  list(pars = exp(rhoOptim), beta_hat = beta_hat, ED = 2*ED1 - ED2, fit = fit, 
        ret = ret, retPrint = retPrint, sigma.sq = diagnostics$sigma.sq, U = U, lin.means = lin.means)
 }
 
